@@ -1,14 +1,3 @@
-def init():
-    from gEconpy.numba_linalg.overloads import (
-        schur_impl,
-        ordqz_impl,
-        qz_impl,
-        solve_continuous_lyapunov_impl,
-        solve_discrete_lyapunov_impl,
-        solve_triangular_impl,
-    )
-
-
 from gEconpy import (
     classes,
     estimation,
