@@ -18,6 +18,7 @@ from scipy.stats import (
 )
 from scipy.stats._distn_infrastructure import rv_frozen
 
+from gEconpy.classes.containers import SymbolDictionary
 from gEconpy.exceptions.exceptions import (
     DistributionOverDefinedException,
     IgnoredCloseMatchWarning,
@@ -98,15 +99,11 @@ DIST_ALIAS_LIST = [
 class CompositeDistribution:
     def __init__(self, dist, **parameters):
         defined_params = {
-            param: value
-            for param, value in parameters.items()
-            if isinstance(value, (int, float))
+            param: value for param, value in parameters.items() if isinstance(value, (int, float))
         }
 
         self.rv_params = {
-            param: value
-            for param, value in parameters.items()
-            if isinstance(value, rv_frozen)
+            param: value for param, value in parameters.items() if isinstance(value, rv_frozen)
         }
         self.d = partial(dist, **defined_params)
 
@@ -120,18 +117,14 @@ class CompositeDistribution:
 
     def _unpack_pdf_dict(self, point_dict):
         param_dict = {
-            param: value
-            for param, value in point_dict.items()
-            if param in self.rv_params.keys()
+            param: value for param, value in point_dict.items() if param in self.rv_params.keys()
         }
         assert set(param_dict.keys()).union(set(self.rv_params.keys())) == set(
             self.rv_params.keys()
         )
 
         point_dict = {
-            param: value
-            for param, value in point_dict.items()
-            if param not in param_dict.keys()
+            param: value for param, value in point_dict.items() if param not in param_dict.keys()
         }
         assert len(point_dict.keys()) == 1
 
@@ -169,9 +162,7 @@ class CompositeDistribution:
         n_samples = len(point_dicts)
         samples = np.zeros(n_samples)
         for idx in range(n_samples):
-            sample_params = {
-                param: value.rvs() for param, value in self.rv_params.items()
-            }
+            sample_params = {param: value.rvs() for param, value in self.rv_params.items()}
             sample_params.update(point_dicts[idx])
             samples[idx] = self.d(**sample_params).rvs()
 
@@ -236,17 +227,13 @@ class BaseDistributionParser(ABC):
         return {canon_name: value}
 
     def _verify_distribution_parameterization(self, param_dict):
-        n_constraints = sum(
-            [self.mean_constraint is not None, self.std_constraint is not None]
-        )
+        n_constraints = sum([self.mean_constraint is not None, self.std_constraint is not None])
         if n_constraints > self.n_params:
             raise InsufficientDegreesOfFreedomException(self.variable_name, self.d_name)
 
         declared_params = list(param_dict.keys())
         boundary_params = [self.lower_bound_param_name, self.upper_bound_param_name]
-        declared_params = [
-            param for param in declared_params if param not in boundary_params
-        ]
+        declared_params = [param for param in declared_params if param not in boundary_params]
 
         n_params_passed = len(declared_params)
         if (n_constraints + n_params_passed) > self.n_params:
@@ -268,10 +255,7 @@ class BaseDistributionParser(ABC):
                     f"{self.std_constraint} > 0",
                 )
 
-        if (
-            self.scale_param_name in declared_params
-            and param_dict[self.scale_param_name] <= 0
-        ):
+        if self.scale_param_name in declared_params and param_dict[self.scale_param_name] <= 0:
             raise InvalidParameterException(
                 self.variable_name,
                 self.d_name,
@@ -321,16 +305,12 @@ class BaseDistributionParser(ABC):
     ) -> Optional[str]:
         candidates = list(set(param_names).intersection(set(aliases)))
         if len(candidates) == 0:
-            invalid_param_names = list(
-                set(param_names) - set(self.all_valid_parameters)
-            )
+            invalid_param_names = list(set(param_names) - set(self.all_valid_parameters))
 
             if len(invalid_param_names) == 0:
                 return None
 
-            best_guess, maybe_typo = find_typos_and_guesses(
-                invalid_param_names, aliases
-            )
+            best_guess, maybe_typo = find_typos_and_guesses(invalid_param_names, aliases)
 
             if best_guess is not None and maybe_typo is not None:
                 warn(
@@ -363,10 +343,7 @@ class BaseDistributionParser(ABC):
             if n_params == 1:
                 message += unused_parameters[0] + "."
             else:
-                message += (
-                    ", ".join(unused_parameters[:-1])
-                    + f", and {unused_parameters[-1]}."
-                )
+                message += ", ".join(unused_parameters[:-1]) + f", and {unused_parameters[-1]}."
             message += " Please verify whether these parameters are needed, and adjust the GCN file accordingly."
 
             warn(message, category=UnusedParameterWarning)
@@ -526,10 +503,7 @@ class NormalDistributionParser(BaseDistributionParser):
                     alpha = (a - loc) / scale
                     beta = (b - loc) / scale
 
-                    return (
-                        truncnorm(loc=loc, scale=scale, a=alpha, b=beta).mean()
-                        - target_mean
-                    )
+                    return truncnorm(loc=loc, scale=scale, a=alpha, b=beta).mean() - target_mean
 
                 loc = optimize.root_scalar(
                     match_mean,
@@ -549,10 +523,7 @@ class NormalDistributionParser(BaseDistributionParser):
                     alpha = (a - loc) / scale
                     beta = (b - loc) / scale
 
-                    return (
-                        truncnorm(loc=loc, scale=scale, a=alpha, b=beta).std()
-                        - target_std
-                    )
+                    return truncnorm(loc=loc, scale=scale, a=alpha, b=beta).std() - target_std
 
                 scale = optimize.root_scalar(
                     match_std,
@@ -755,16 +726,12 @@ class UniformDistributionParser(BaseDistributionParser):
 
             elif self.scale_param_name in parameters:
                 # TODO: Determine if this case is plausible, doesn't seem so, because sigma = 12 ** (-1/2) * scale
-                raise ValueError(
-                    "Scale and Std are not enough to identify a Uniform distribution!"
-                )
+                raise ValueError("Scale and Std are not enough to identify a Uniform distribution!")
 
         # Case 4: User passed the bounds directly, convert to loc and scale then delete
         else:
             if self.lower_bound_param_name in parameters:
-                param_dict[self.loc_param_name] = param_dict[
-                    self.lower_bound_param_name
-                ]
+                param_dict[self.loc_param_name] = param_dict[self.lower_bound_param_name]
                 del param_dict[self.lower_bound_param_name]
             if self.upper_bound_param_name in parameters:
                 b = param_dict[self.upper_bound_param_name]
@@ -787,9 +754,7 @@ class InverseGammaDistributionParser(BaseDistributionParser):
             upper_bound_param_name=None,
             lower_bound_param_name=None,
             n_params=3,
-            all_valid_parameters=INV_GAMMA_SHAPE_ALIASES
-            + INV_GAMMA_SCALE_ALIASES
-            + ["loc"],
+            all_valid_parameters=INV_GAMMA_SHAPE_ALIASES + INV_GAMMA_SCALE_ALIASES + ["loc"],
         )
 
     def build_distribution(
@@ -986,9 +951,9 @@ class BetaDistributionParser(BaseDistributionParser):
                 )
 
             x = (1 - mean) / mean
-            param_dict[self.shape_param_name_1] = alpha = x / (
-                std**2 * (x + 1) ** 3
-            ) - (1 / (x + 1))
+            param_dict[self.shape_param_name_1] = alpha = x / (std**2 * (x + 1) ** 3) - (
+                1 / (x + 1)
+            )
             param_dict[self.shape_param_name_2] = alpha * x
 
         # # Case 2: No moment constraints
@@ -1050,10 +1015,7 @@ class GammaDistributionParser(BaseDistributionParser):
             lower_bound_param_name=None,
             upper_bound_param_name=None,
             n_params=3,
-            all_valid_parameters=GAMMA_SHAPE_ALIASES
-            + GAMMA_SCALE_ALIASES
-            + MOMENTS
-            + ["loc"],
+            all_valid_parameters=GAMMA_SHAPE_ALIASES + GAMMA_SCALE_ALIASES + MOMENTS + ["loc"],
         )
 
     def build_distribution(
@@ -1189,9 +1151,7 @@ def build_alias_to_canon_dict(alias_list, cannon_names) -> Dict[str, str]:
     return alias_to_canon_dict
 
 
-def preprocess_distribution_string(
-    variable_name: str, d_string: str
-) -> Tuple[str, Dict[str, str]]:
+def preprocess_distribution_string(variable_name: str, d_string: str) -> Tuple[str, Dict[str, str]]:
     """
     Parameters
     ----------
@@ -1215,9 +1175,7 @@ def preprocess_distribution_string(
     dist_name_pattern = r"(\w+)"
     not_last_arg_pattern = rf"(\w+ ?={general_pattern}, ?)"
     last_arg_pattern = rf"(\w+ ?={general_pattern})"
-    valid_pattern = (
-        rf"{dist_name_pattern}\({not_last_arg_pattern}*?{last_arg_pattern}\),?$"
-    )
+    valid_pattern = rf"{dist_name_pattern}\({not_last_arg_pattern}*?{last_arg_pattern}\),?$"
 
     # TODO: sort out where the typo is and tell the user.
     if re.search(valid_pattern, d_string) is None:
@@ -1348,9 +1306,7 @@ def rename_dict_keys_with_value_transform(
     result = {}
     matches = [key for key in d.keys() if key in set(to_rename)]
     if len(matches) > 1:
-        raise MultipleParameterDefinitionException(
-            variable_name, d_name, new_key, matches
-        )
+        raise MultipleParameterDefinitionException(variable_name, d_name, new_key, matches)
 
     for key, value in d.items():
         if key in to_rename:
@@ -1372,9 +1328,7 @@ def param_values_to_floats(param_dict: Dict):
 
 def split_out_composite_distributions(
     variable_names: List[str], d_names: List[str], param_dicts: List[Dict[str, str]]
-) -> Tuple[
-    Dict[str, Tuple[str, Dict[str, str]]], Dict[str, Tuple[str, Dict[str, str]]]
-]:
+) -> Tuple[Dict[str, Tuple[str, Dict[str, str]]], Dict[str, Tuple[str, Dict[str, str]]]]:
     basic_distributions = {}
     composite_distributions = {}
 
@@ -1395,9 +1349,7 @@ def fetch_rv_params(param_dict, model):
         elif isinstance(v, str):
             return_dict[k] = model[v]
         else:
-            raise ValueError(
-                f"Found an illegal key:value pair in prior param dict, {k}:{v}"
-            )
+            raise ValueError(f"Found an illegal key:value pair in prior param dict, {k}:{v}")
 
     return return_dict
 
@@ -1447,12 +1399,8 @@ def composite_distribution_factory(
 
     # validate parameters by simple rename, error on more complicated setups (no moment constraints!)
     if d_name == "normal":
-        has_upper_bound = any(
-            [x in set(param_dict.keys()) for x in UPPER_BOUND_ALIASES]
-        )
-        has_lower_bound = any(
-            [x in set(param_dict.keys()) for x in LOWER_BOUND_ALIASES]
-        )
+        has_upper_bound = any([x in set(param_dict.keys()) for x in UPPER_BOUND_ALIASES])
+        has_lower_bound = any([x in set(param_dict.keys()) for x in LOWER_BOUND_ALIASES])
 
         if (has_upper_bound or has_lower_bound) and package == "scipy":
             warn(
@@ -1554,19 +1502,15 @@ def composite_distribution_factory(
         return d
 
 
-def create_prior_distribution_dictionary(
-    raw_prior_dict: Dict[str, str]
-) -> Dict[str, Any]:
+def create_prior_distribution_dictionary(raw_prior_dict: Dict[str, str]) -> Dict[str, Any]:
     variable_names, d_names, param_dicts = preprocess_prior_dict(raw_prior_dict)
     basic_distributions, compound_distributions = split_out_composite_distributions(
         variable_names, d_names, param_dicts
     )
-    prior_dict = {}
+    prior_dict = SymbolDictionary()
 
     for variable_name, (d_name, param_dict) in basic_distributions.items():
-        d = distribution_factory(
-            variable_name=variable_name, d_name=d_name, param_dict=param_dict
-        )
+        d = distribution_factory(variable_name=variable_name, d_name=d_name, param_dict=param_dict)
         prior_dict[variable_name] = d
 
     for variable_name, (d_name, param_dict) in compound_distributions.items():
